# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.bill_data import BillData
from ..types.entrypointfield import Entrypointfield
from ..types.force_customer_creation import ForceCustomerCreation
from ..types.idempotency_key import IdempotencyKey
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payment_detail import PaymentDetail
from ..types.payor_data_request import PayorDataRequest
from ..types.schedule_detail import ScheduleDetail
from ..types.source import Source
from ..types.subdomain import Subdomain
from ..types.subscription_query_records import SubscriptionQueryRecords
from .types.add_subscription_response import AddSubscriptionResponse
from .types.remove_subscription_response import RemoveSubscriptionResponse
from .types.request_schedule_payment_method import RequestSchedulePaymentMethod
from .types.set_pause import SetPause
from .types.update_subscription_response import UpdateSubscriptionResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSubscriptionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_subscription(
        self, sub_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SubscriptionQueryRecords]:
        """
        Retrieves a single subscription's details.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SubscriptionQueryRecords]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Subscription/{jsonable_encoder(sub_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubscriptionQueryRecords,
                    parse_obj_as(
                        type_=SubscriptionQueryRecords,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def new_subscription(
        self,
        *,
        force_customer_creation: typing.Optional[ForceCustomerCreation] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        invoice_data: typing.Optional[BillData] = OMIT,
        payment_details: typing.Optional[PaymentDetail] = OMIT,
        payment_method: typing.Optional[RequestSchedulePaymentMethod] = OMIT,
        schedule_details: typing.Optional[ScheduleDetail] = OMIT,
        set_pause: typing.Optional[SetPause] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddSubscriptionResponse]:
        """
        Creates a subscription or scheduled payment to run at a specified time and frequency.

        Parameters
        ----------
        force_customer_creation : typing.Optional[ForceCustomerCreation]

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the customer/payor.

        entry_point : typing.Optional[Entrypointfield]

        invoice_data : typing.Optional[BillData]
            Object describing an Invoice linked to the subscription.

        payment_details : typing.Optional[PaymentDetail]
            Object describing details of the payment. To skip the payment, set the `totalAmount` to 0. Payments will be paused until the amount is updated to a non-zero value. When `totalAmount` is set to 0, the `serviceFee` must also be set to 0.

        payment_method : typing.Optional[RequestSchedulePaymentMethod]
            Information about the payment method for the transaction. Required and recommended fields for each payment method type are described in each schema below.

        schedule_details : typing.Optional[ScheduleDetail]
            Object describing the schedule for subscription.

        set_pause : typing.Optional[SetPause]

        source : typing.Optional[Source]

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "Subscription/add",
            method="POST",
            params={
                "forceCustomerCreation": force_customer_creation,
            },
            json={
                "customerData": convert_and_respect_annotation_metadata(
                    object_=customer_data, annotation=PayorDataRequest, direction="write"
                ),
                "entryPoint": entry_point,
                "invoiceData": convert_and_respect_annotation_metadata(
                    object_=invoice_data, annotation=BillData, direction="write"
                ),
                "paymentDetails": convert_and_respect_annotation_metadata(
                    object_=payment_details, annotation=PaymentDetail, direction="write"
                ),
                "paymentMethod": convert_and_respect_annotation_metadata(
                    object_=payment_method, annotation=RequestSchedulePaymentMethod, direction="write"
                ),
                "scheduleDetails": convert_and_respect_annotation_metadata(
                    object_=schedule_details, annotation=ScheduleDetail, direction="write"
                ),
                "setPause": set_pause,
                "source": source,
                "subdomain": subdomain,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddSubscriptionResponse,
                    parse_obj_as(
                        type_=AddSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def remove_subscription(
        self, sub_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RemoveSubscriptionResponse]:
        """
        Deletes a subscription, autopay, or recurring payment and prevents future charges.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RemoveSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Subscription/{jsonable_encoder(sub_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RemoveSubscriptionResponse,
                    parse_obj_as(
                        type_=RemoveSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_subscription(
        self,
        sub_id: int,
        *,
        payment_details: typing.Optional[PaymentDetail] = OMIT,
        schedule_details: typing.Optional[ScheduleDetail] = OMIT,
        set_pause: typing.Optional[SetPause] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateSubscriptionResponse]:
        """
        Updates a subscription's details.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        payment_details : typing.Optional[PaymentDetail]
            Object describing details of the payment. To skip the payment, set the `totalAmount` to 0. Payments will be paused until the amount is updated to a non-zero value. When `totalAmount` is set to 0, the `serviceFee` must also be set to 0.

        schedule_details : typing.Optional[ScheduleDetail]
            Object describing the schedule for subscription

        set_pause : typing.Optional[SetPause]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Subscription/{jsonable_encoder(sub_id)}",
            method="PUT",
            json={
                "paymentDetails": convert_and_respect_annotation_metadata(
                    object_=payment_details, annotation=PaymentDetail, direction="write"
                ),
                "scheduleDetails": convert_and_respect_annotation_metadata(
                    object_=schedule_details, annotation=ScheduleDetail, direction="write"
                ),
                "setPause": set_pause,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateSubscriptionResponse,
                    parse_obj_as(
                        type_=UpdateSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSubscriptionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_subscription(
        self, sub_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SubscriptionQueryRecords]:
        """
        Retrieves a single subscription's details.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SubscriptionQueryRecords]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Subscription/{jsonable_encoder(sub_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubscriptionQueryRecords,
                    parse_obj_as(
                        type_=SubscriptionQueryRecords,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def new_subscription(
        self,
        *,
        force_customer_creation: typing.Optional[ForceCustomerCreation] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        invoice_data: typing.Optional[BillData] = OMIT,
        payment_details: typing.Optional[PaymentDetail] = OMIT,
        payment_method: typing.Optional[RequestSchedulePaymentMethod] = OMIT,
        schedule_details: typing.Optional[ScheduleDetail] = OMIT,
        set_pause: typing.Optional[SetPause] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddSubscriptionResponse]:
        """
        Creates a subscription or scheduled payment to run at a specified time and frequency.

        Parameters
        ----------
        force_customer_creation : typing.Optional[ForceCustomerCreation]

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the customer/payor.

        entry_point : typing.Optional[Entrypointfield]

        invoice_data : typing.Optional[BillData]
            Object describing an Invoice linked to the subscription.

        payment_details : typing.Optional[PaymentDetail]
            Object describing details of the payment. To skip the payment, set the `totalAmount` to 0. Payments will be paused until the amount is updated to a non-zero value. When `totalAmount` is set to 0, the `serviceFee` must also be set to 0.

        payment_method : typing.Optional[RequestSchedulePaymentMethod]
            Information about the payment method for the transaction. Required and recommended fields for each payment method type are described in each schema below.

        schedule_details : typing.Optional[ScheduleDetail]
            Object describing the schedule for subscription.

        set_pause : typing.Optional[SetPause]

        source : typing.Optional[Source]

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "Subscription/add",
            method="POST",
            params={
                "forceCustomerCreation": force_customer_creation,
            },
            json={
                "customerData": convert_and_respect_annotation_metadata(
                    object_=customer_data, annotation=PayorDataRequest, direction="write"
                ),
                "entryPoint": entry_point,
                "invoiceData": convert_and_respect_annotation_metadata(
                    object_=invoice_data, annotation=BillData, direction="write"
                ),
                "paymentDetails": convert_and_respect_annotation_metadata(
                    object_=payment_details, annotation=PaymentDetail, direction="write"
                ),
                "paymentMethod": convert_and_respect_annotation_metadata(
                    object_=payment_method, annotation=RequestSchedulePaymentMethod, direction="write"
                ),
                "scheduleDetails": convert_and_respect_annotation_metadata(
                    object_=schedule_details, annotation=ScheduleDetail, direction="write"
                ),
                "setPause": set_pause,
                "source": source,
                "subdomain": subdomain,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddSubscriptionResponse,
                    parse_obj_as(
                        type_=AddSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def remove_subscription(
        self, sub_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RemoveSubscriptionResponse]:
        """
        Deletes a subscription, autopay, or recurring payment and prevents future charges.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RemoveSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Subscription/{jsonable_encoder(sub_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RemoveSubscriptionResponse,
                    parse_obj_as(
                        type_=RemoveSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_subscription(
        self,
        sub_id: int,
        *,
        payment_details: typing.Optional[PaymentDetail] = OMIT,
        schedule_details: typing.Optional[ScheduleDetail] = OMIT,
        set_pause: typing.Optional[SetPause] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateSubscriptionResponse]:
        """
        Updates a subscription's details.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        payment_details : typing.Optional[PaymentDetail]
            Object describing details of the payment. To skip the payment, set the `totalAmount` to 0. Payments will be paused until the amount is updated to a non-zero value. When `totalAmount` is set to 0, the `serviceFee` must also be set to 0.

        schedule_details : typing.Optional[ScheduleDetail]
            Object describing the schedule for subscription

        set_pause : typing.Optional[SetPause]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Subscription/{jsonable_encoder(sub_id)}",
            method="PUT",
            json={
                "paymentDetails": convert_and_respect_annotation_metadata(
                    object_=payment_details, annotation=PaymentDetail, direction="write"
                ),
                "scheduleDetails": convert_and_respect_annotation_metadata(
                    object_=schedule_details, annotation=ScheduleDetail, direction="write"
                ),
                "setPause": set_pause,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateSubscriptionResponse,
                    parse_obj_as(
                        type_=UpdateSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.contact_element import ContactElement
from ..types.element import Element
from ..types.entry import Entry
from ..types.header_element import HeaderElement
from ..types.idempotency_key import IdempotencyKey
from ..types.invoice_element import InvoiceElement
from ..types.label_element import LabelElement
from ..types.method_element import MethodElement
from ..types.note_element import NoteElement
from ..types.page_element import PageElement
from ..types.pagelink_setting import PagelinkSetting
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payor_element import PayorElement
from ..types.push_pay_link_request import PushPayLinkRequest
from .types.get_pay_link_from_id_response import GetPayLinkFromIdResponse
from .types.payabli_api_response_payment_links import PayabliApiResponsePaymentLinks

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPaymentLinkClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_pay_link_from_invoice(
        self,
        id_invoice: int,
        *,
        amount_fixed: typing.Optional[bool] = None,
        mail_2: typing.Optional[str] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        contact_us: typing.Optional[ContactElement] = OMIT,
        invoices: typing.Optional[InvoiceElement] = OMIT,
        logo: typing.Optional[Element] = OMIT,
        message_before_paying: typing.Optional[LabelElement] = OMIT,
        notes: typing.Optional[NoteElement] = OMIT,
        page: typing.Optional[PageElement] = OMIT,
        payment_button: typing.Optional[LabelElement] = OMIT,
        payment_methods: typing.Optional[MethodElement] = OMIT,
        payor: typing.Optional[PayorElement] = OMIT,
        review: typing.Optional[HeaderElement] = OMIT,
        settings: typing.Optional[PagelinkSetting] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Generates a payment link for an invoice from the invoice ID.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        amount_fixed : typing.Optional[bool]
            Indicates whether customer can modify the payment amount. A value of `true` means the amount isn't modifiable, a value `false` means the payor can modify the amount to pay.

        mail_2 : typing.Optional[str]
            List of recipient email addresses. When there is more than one, separate them by a semicolon (;).

        idempotency_key : typing.Optional[IdempotencyKey]

        contact_us : typing.Optional[ContactElement]
            ContactUs section of payment link page

        invoices : typing.Optional[InvoiceElement]
            Invoices section of payment link page

        logo : typing.Optional[Element]
            Logo section of payment link page

        message_before_paying : typing.Optional[LabelElement]
            Message section of payment link page

        notes : typing.Optional[NoteElement]
            Notes section of payment link page

        page : typing.Optional[PageElement]
            Page header section of payment link page

        payment_button : typing.Optional[LabelElement]
            Payment button section of payment link page

        payment_methods : typing.Optional[MethodElement]
            Payment methods section of payment link page

        payor : typing.Optional[PayorElement]
            Customer/Payor section of payment link page

        review : typing.Optional[HeaderElement]
            Review section of payment link page

        settings : typing.Optional[PagelinkSetting]
            Settings section of payment link page

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentLink/{jsonable_encoder(id_invoice)}",
            method="POST",
            params={
                "amountFixed": amount_fixed,
                "mail2": mail_2,
            },
            json={
                "contactUs": convert_and_respect_annotation_metadata(
                    object_=contact_us, annotation=ContactElement, direction="write"
                ),
                "invoices": convert_and_respect_annotation_metadata(
                    object_=invoices, annotation=InvoiceElement, direction="write"
                ),
                "logo": convert_and_respect_annotation_metadata(object_=logo, annotation=Element, direction="write"),
                "messageBeforePaying": convert_and_respect_annotation_metadata(
                    object_=message_before_paying, annotation=LabelElement, direction="write"
                ),
                "notes": convert_and_respect_annotation_metadata(
                    object_=notes, annotation=NoteElement, direction="write"
                ),
                "page": convert_and_respect_annotation_metadata(
                    object_=page, annotation=PageElement, direction="write"
                ),
                "paymentButton": convert_and_respect_annotation_metadata(
                    object_=payment_button, annotation=LabelElement, direction="write"
                ),
                "paymentMethods": convert_and_respect_annotation_metadata(
                    object_=payment_methods, annotation=MethodElement, direction="write"
                ),
                "payor": convert_and_respect_annotation_metadata(
                    object_=payor, annotation=PayorElement, direction="write"
                ),
                "review": convert_and_respect_annotation_metadata(
                    object_=review, annotation=HeaderElement, direction="write"
                ),
                "settings": convert_and_respect_annotation_metadata(
                    object_=settings, annotation=PagelinkSetting, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def add_pay_link_from_bill(
        self,
        bill_id: int,
        *,
        amount_fixed: typing.Optional[bool] = None,
        mail_2: typing.Optional[str] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        contact_us: typing.Optional[ContactElement] = OMIT,
        invoices: typing.Optional[InvoiceElement] = OMIT,
        logo: typing.Optional[Element] = OMIT,
        message_before_paying: typing.Optional[LabelElement] = OMIT,
        notes: typing.Optional[NoteElement] = OMIT,
        page: typing.Optional[PageElement] = OMIT,
        payment_button: typing.Optional[LabelElement] = OMIT,
        payment_methods: typing.Optional[MethodElement] = OMIT,
        payor: typing.Optional[PayorElement] = OMIT,
        review: typing.Optional[HeaderElement] = OMIT,
        settings: typing.Optional[PagelinkSetting] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Generates a payment link for a bill from the bill ID.

        Parameters
        ----------
        bill_id : int
            The Payabli ID for the bill.

        amount_fixed : typing.Optional[bool]
            Indicates whether customer can modify the payment amount. A value of `true` means the amount isn't modifiable, a value `false` means the payor can modify the amount to pay.

        mail_2 : typing.Optional[str]
            List of recipient email addresses. When there is more than one, separate them by a semicolon (;).

        idempotency_key : typing.Optional[IdempotencyKey]

        contact_us : typing.Optional[ContactElement]
            ContactUs section of payment link page

        invoices : typing.Optional[InvoiceElement]
            Invoices section of payment link page

        logo : typing.Optional[Element]
            Logo section of payment link page

        message_before_paying : typing.Optional[LabelElement]
            Message section of payment link page

        notes : typing.Optional[NoteElement]
            Notes section of payment link page

        page : typing.Optional[PageElement]
            Page header section of payment link page

        payment_button : typing.Optional[LabelElement]
            Payment button section of payment link page

        payment_methods : typing.Optional[MethodElement]
            Payment methods section of payment link page

        payor : typing.Optional[PayorElement]
            Customer/Payor section of payment link page

        review : typing.Optional[HeaderElement]
            Review section of payment link page

        settings : typing.Optional[PagelinkSetting]
            Settings section of payment link page

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentLink/bill/{jsonable_encoder(bill_id)}",
            method="POST",
            params={
                "amountFixed": amount_fixed,
                "mail2": mail_2,
            },
            json={
                "contactUs": convert_and_respect_annotation_metadata(
                    object_=contact_us, annotation=ContactElement, direction="write"
                ),
                "invoices": convert_and_respect_annotation_metadata(
                    object_=invoices, annotation=InvoiceElement, direction="write"
                ),
                "logo": convert_and_respect_annotation_metadata(object_=logo, annotation=Element, direction="write"),
                "messageBeforePaying": convert_and_respect_annotation_metadata(
                    object_=message_before_paying, annotation=LabelElement, direction="write"
                ),
                "notes": convert_and_respect_annotation_metadata(
                    object_=notes, annotation=NoteElement, direction="write"
                ),
                "page": convert_and_respect_annotation_metadata(
                    object_=page, annotation=PageElement, direction="write"
                ),
                "paymentButton": convert_and_respect_annotation_metadata(
                    object_=payment_button, annotation=LabelElement, direction="write"
                ),
                "paymentMethods": convert_and_respect_annotation_metadata(
                    object_=payment_methods, annotation=MethodElement, direction="write"
                ),
                "payor": convert_and_respect_annotation_metadata(
                    object_=payor, annotation=PayorElement, direction="write"
                ),
                "review": convert_and_respect_annotation_metadata(
                    object_=review, annotation=HeaderElement, direction="write"
                ),
                "settings": convert_and_respect_annotation_metadata(
                    object_=settings, annotation=PagelinkSetting, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_pay_link_from_id(
        self, pay_link_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Deletes a payment link by ID.

        Parameters
        ----------
        pay_link_id : str
            ID for the payment link.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentLink/{jsonable_encoder(pay_link_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_pay_link_from_id(
        self, paylink_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetPayLinkFromIdResponse]:
        """
        Retrieves a payment link by ID.

        Parameters
        ----------
        paylink_id : str
            ID for payment link

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetPayLinkFromIdResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentLink/load/{jsonable_encoder(paylink_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetPayLinkFromIdResponse,
                    parse_obj_as(
                        type_=GetPayLinkFromIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def push_pay_link_from_id(
        self, pay_link_id: str, *, request: PushPayLinkRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Send a payment link to the specified email addresses or phone numbers.

        Parameters
        ----------
        pay_link_id : str
            ID for the payment link.

        request : PushPayLinkRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentLink/push/{jsonable_encoder(pay_link_id)}",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=PushPayLinkRequest, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def refresh_pay_link_from_id(
        self,
        pay_link_id: str,
        *,
        amount_fixed: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Refresh a payment link's content after an update.

        Parameters
        ----------
        pay_link_id : str
            ID for the payment link.

        amount_fixed : typing.Optional[bool]
            Indicates whether customer can modify the payment amount. A value of `true` means the amount isn't modifiable, a value `false` means the payor can modify the amount to pay.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentLink/refresh/{jsonable_encoder(pay_link_id)}",
            method="GET",
            params={
                "amountFixed": amount_fixed,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def send_pay_link_from_id(
        self,
        pay_link_id: str,
        *,
        attachfile: typing.Optional[bool] = None,
        mail_2: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Sends a payment link to the specified email addresses.

        Parameters
        ----------
        pay_link_id : str
            ID for the payment link.

        attachfile : typing.Optional[bool]
            When `true`, attaches a PDF version of invoice to the email.

        mail_2 : typing.Optional[str]
            List of recipient email addresses. When there is more than one, separate them by a semicolon (;).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentLink/send/{jsonable_encoder(pay_link_id)}",
            method="GET",
            params={
                "attachfile": attachfile,
                "mail2": mail_2,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_pay_link_from_id(
        self,
        pay_link_id: str,
        *,
        contact_us: typing.Optional[ContactElement] = OMIT,
        logo: typing.Optional[Element] = OMIT,
        message_before_paying: typing.Optional[LabelElement] = OMIT,
        notes: typing.Optional[NoteElement] = OMIT,
        page: typing.Optional[PageElement] = OMIT,
        payment_button: typing.Optional[LabelElement] = OMIT,
        payment_methods: typing.Optional[MethodElement] = OMIT,
        review: typing.Optional[HeaderElement] = OMIT,
        settings: typing.Optional[PagelinkSetting] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Updates a payment link's details.

        Parameters
        ----------
        pay_link_id : str
            ID for the payment link.

        contact_us : typing.Optional[ContactElement]
            ContactUs section of payment link page

        logo : typing.Optional[Element]
            Logo section of payment link page

        message_before_paying : typing.Optional[LabelElement]
            Message section of payment link page

        notes : typing.Optional[NoteElement]
            Notes section of payment link page

        page : typing.Optional[PageElement]
            Page header section of payment link page

        payment_button : typing.Optional[LabelElement]
            Payment button section of payment link page

        payment_methods : typing.Optional[MethodElement]
            Payment methods section of payment link page

        review : typing.Optional[HeaderElement]
            Review section of payment link page

        settings : typing.Optional[PagelinkSetting]
            Settings section of payment link page

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentLink/update/{jsonable_encoder(pay_link_id)}",
            method="PUT",
            json={
                "contactUs": convert_and_respect_annotation_metadata(
                    object_=contact_us, annotation=ContactElement, direction="write"
                ),
                "logo": convert_and_respect_annotation_metadata(object_=logo, annotation=Element, direction="write"),
                "messageBeforePaying": convert_and_respect_annotation_metadata(
                    object_=message_before_paying, annotation=LabelElement, direction="write"
                ),
                "notes": convert_and_respect_annotation_metadata(
                    object_=notes, annotation=NoteElement, direction="write"
                ),
                "page": convert_and_respect_annotation_metadata(
                    object_=page, annotation=PageElement, direction="write"
                ),
                "paymentButton": convert_and_respect_annotation_metadata(
                    object_=payment_button, annotation=LabelElement, direction="write"
                ),
                "paymentMethods": convert_and_respect_annotation_metadata(
                    object_=payment_methods, annotation=MethodElement, direction="write"
                ),
                "review": convert_and_respect_annotation_metadata(
                    object_=review, annotation=HeaderElement, direction="write"
                ),
                "settings": convert_and_respect_annotation_metadata(
                    object_=settings, annotation=PagelinkSetting, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def add_pay_link_from_bill_lot_number(
        self,
        lot_number: str,
        *,
        entry_point: Entry,
        vendor_number: str,
        mail_2: typing.Optional[str] = None,
        amount_fixed: typing.Optional[str] = None,
        contact_us: typing.Optional[ContactElement] = OMIT,
        invoices: typing.Optional[InvoiceElement] = OMIT,
        logo: typing.Optional[Element] = OMIT,
        message_before_paying: typing.Optional[LabelElement] = OMIT,
        notes: typing.Optional[NoteElement] = OMIT,
        page: typing.Optional[PageElement] = OMIT,
        payment_button: typing.Optional[LabelElement] = OMIT,
        payment_methods: typing.Optional[MethodElement] = OMIT,
        payor: typing.Optional[PayorElement] = OMIT,
        review: typing.Optional[HeaderElement] = OMIT,
        settings: typing.Optional[PagelinkSetting] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Generates a vendor payment link for a specific bill lot number. This allows you to pay all bills with the same lot number for a vendor with a single payment link.

        Parameters
        ----------
        lot_number : str
            Lot number of the bills to pay. All bills with this lot number will be included.

        entry_point : Entry

        vendor_number : str
            The vendor number for the vendor being paid with this payment link.

        mail_2 : typing.Optional[str]
            List of recipient email addresses. When there is more than one, separate them by a semicolon (;).

        amount_fixed : typing.Optional[str]
            Indicates whether customer can modify the payment amount. A value of `true` means the amount isn't modifiable, a value `false` means the payor can modify the amount to pay.

        contact_us : typing.Optional[ContactElement]
            ContactUs section of payment link page

        invoices : typing.Optional[InvoiceElement]
            Invoices section of payment link page

        logo : typing.Optional[Element]
            Logo section of payment link page

        message_before_paying : typing.Optional[LabelElement]
            Message section of payment link page

        notes : typing.Optional[NoteElement]
            Notes section of payment link page

        page : typing.Optional[PageElement]
            Page header section of payment link page

        payment_button : typing.Optional[LabelElement]
            Payment button section of payment link page

        payment_methods : typing.Optional[MethodElement]
            Payment methods section of payment link page

        payor : typing.Optional[PayorElement]
            Customer/Payor section of payment link page

        review : typing.Optional[HeaderElement]
            Review section of payment link page

        settings : typing.Optional[PagelinkSetting]
            Settings section of payment link page

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"PaymentLink/bill/lotNumber/{jsonable_encoder(lot_number)}",
            method="POST",
            params={
                "entryPoint": entry_point,
                "vendorNumber": vendor_number,
                "mail2": mail_2,
                "amountFixed": amount_fixed,
            },
            json={
                "contactUs": convert_and_respect_annotation_metadata(
                    object_=contact_us, annotation=ContactElement, direction="write"
                ),
                "invoices": convert_and_respect_annotation_metadata(
                    object_=invoices, annotation=InvoiceElement, direction="write"
                ),
                "logo": convert_and_respect_annotation_metadata(object_=logo, annotation=Element, direction="write"),
                "messageBeforePaying": convert_and_respect_annotation_metadata(
                    object_=message_before_paying, annotation=LabelElement, direction="write"
                ),
                "notes": convert_and_respect_annotation_metadata(
                    object_=notes, annotation=NoteElement, direction="write"
                ),
                "page": convert_and_respect_annotation_metadata(
                    object_=page, annotation=PageElement, direction="write"
                ),
                "paymentButton": convert_and_respect_annotation_metadata(
                    object_=payment_button, annotation=LabelElement, direction="write"
                ),
                "paymentMethods": convert_and_respect_annotation_metadata(
                    object_=payment_methods, annotation=MethodElement, direction="write"
                ),
                "payor": convert_and_respect_annotation_metadata(
                    object_=payor, annotation=PayorElement, direction="write"
                ),
                "review": convert_and_respect_annotation_metadata(
                    object_=review, annotation=HeaderElement, direction="write"
                ),
                "settings": convert_and_respect_annotation_metadata(
                    object_=settings, annotation=PagelinkSetting, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPaymentLinkClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_pay_link_from_invoice(
        self,
        id_invoice: int,
        *,
        amount_fixed: typing.Optional[bool] = None,
        mail_2: typing.Optional[str] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        contact_us: typing.Optional[ContactElement] = OMIT,
        invoices: typing.Optional[InvoiceElement] = OMIT,
        logo: typing.Optional[Element] = OMIT,
        message_before_paying: typing.Optional[LabelElement] = OMIT,
        notes: typing.Optional[NoteElement] = OMIT,
        page: typing.Optional[PageElement] = OMIT,
        payment_button: typing.Optional[LabelElement] = OMIT,
        payment_methods: typing.Optional[MethodElement] = OMIT,
        payor: typing.Optional[PayorElement] = OMIT,
        review: typing.Optional[HeaderElement] = OMIT,
        settings: typing.Optional[PagelinkSetting] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Generates a payment link for an invoice from the invoice ID.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        amount_fixed : typing.Optional[bool]
            Indicates whether customer can modify the payment amount. A value of `true` means the amount isn't modifiable, a value `false` means the payor can modify the amount to pay.

        mail_2 : typing.Optional[str]
            List of recipient email addresses. When there is more than one, separate them by a semicolon (;).

        idempotency_key : typing.Optional[IdempotencyKey]

        contact_us : typing.Optional[ContactElement]
            ContactUs section of payment link page

        invoices : typing.Optional[InvoiceElement]
            Invoices section of payment link page

        logo : typing.Optional[Element]
            Logo section of payment link page

        message_before_paying : typing.Optional[LabelElement]
            Message section of payment link page

        notes : typing.Optional[NoteElement]
            Notes section of payment link page

        page : typing.Optional[PageElement]
            Page header section of payment link page

        payment_button : typing.Optional[LabelElement]
            Payment button section of payment link page

        payment_methods : typing.Optional[MethodElement]
            Payment methods section of payment link page

        payor : typing.Optional[PayorElement]
            Customer/Payor section of payment link page

        review : typing.Optional[HeaderElement]
            Review section of payment link page

        settings : typing.Optional[PagelinkSetting]
            Settings section of payment link page

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentLink/{jsonable_encoder(id_invoice)}",
            method="POST",
            params={
                "amountFixed": amount_fixed,
                "mail2": mail_2,
            },
            json={
                "contactUs": convert_and_respect_annotation_metadata(
                    object_=contact_us, annotation=ContactElement, direction="write"
                ),
                "invoices": convert_and_respect_annotation_metadata(
                    object_=invoices, annotation=InvoiceElement, direction="write"
                ),
                "logo": convert_and_respect_annotation_metadata(object_=logo, annotation=Element, direction="write"),
                "messageBeforePaying": convert_and_respect_annotation_metadata(
                    object_=message_before_paying, annotation=LabelElement, direction="write"
                ),
                "notes": convert_and_respect_annotation_metadata(
                    object_=notes, annotation=NoteElement, direction="write"
                ),
                "page": convert_and_respect_annotation_metadata(
                    object_=page, annotation=PageElement, direction="write"
                ),
                "paymentButton": convert_and_respect_annotation_metadata(
                    object_=payment_button, annotation=LabelElement, direction="write"
                ),
                "paymentMethods": convert_and_respect_annotation_metadata(
                    object_=payment_methods, annotation=MethodElement, direction="write"
                ),
                "payor": convert_and_respect_annotation_metadata(
                    object_=payor, annotation=PayorElement, direction="write"
                ),
                "review": convert_and_respect_annotation_metadata(
                    object_=review, annotation=HeaderElement, direction="write"
                ),
                "settings": convert_and_respect_annotation_metadata(
                    object_=settings, annotation=PagelinkSetting, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def add_pay_link_from_bill(
        self,
        bill_id: int,
        *,
        amount_fixed: typing.Optional[bool] = None,
        mail_2: typing.Optional[str] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        contact_us: typing.Optional[ContactElement] = OMIT,
        invoices: typing.Optional[InvoiceElement] = OMIT,
        logo: typing.Optional[Element] = OMIT,
        message_before_paying: typing.Optional[LabelElement] = OMIT,
        notes: typing.Optional[NoteElement] = OMIT,
        page: typing.Optional[PageElement] = OMIT,
        payment_button: typing.Optional[LabelElement] = OMIT,
        payment_methods: typing.Optional[MethodElement] = OMIT,
        payor: typing.Optional[PayorElement] = OMIT,
        review: typing.Optional[HeaderElement] = OMIT,
        settings: typing.Optional[PagelinkSetting] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Generates a payment link for a bill from the bill ID.

        Parameters
        ----------
        bill_id : int
            The Payabli ID for the bill.

        amount_fixed : typing.Optional[bool]
            Indicates whether customer can modify the payment amount. A value of `true` means the amount isn't modifiable, a value `false` means the payor can modify the amount to pay.

        mail_2 : typing.Optional[str]
            List of recipient email addresses. When there is more than one, separate them by a semicolon (;).

        idempotency_key : typing.Optional[IdempotencyKey]

        contact_us : typing.Optional[ContactElement]
            ContactUs section of payment link page

        invoices : typing.Optional[InvoiceElement]
            Invoices section of payment link page

        logo : typing.Optional[Element]
            Logo section of payment link page

        message_before_paying : typing.Optional[LabelElement]
            Message section of payment link page

        notes : typing.Optional[NoteElement]
            Notes section of payment link page

        page : typing.Optional[PageElement]
            Page header section of payment link page

        payment_button : typing.Optional[LabelElement]
            Payment button section of payment link page

        payment_methods : typing.Optional[MethodElement]
            Payment methods section of payment link page

        payor : typing.Optional[PayorElement]
            Customer/Payor section of payment link page

        review : typing.Optional[HeaderElement]
            Review section of payment link page

        settings : typing.Optional[PagelinkSetting]
            Settings section of payment link page

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentLink/bill/{jsonable_encoder(bill_id)}",
            method="POST",
            params={
                "amountFixed": amount_fixed,
                "mail2": mail_2,
            },
            json={
                "contactUs": convert_and_respect_annotation_metadata(
                    object_=contact_us, annotation=ContactElement, direction="write"
                ),
                "invoices": convert_and_respect_annotation_metadata(
                    object_=invoices, annotation=InvoiceElement, direction="write"
                ),
                "logo": convert_and_respect_annotation_metadata(object_=logo, annotation=Element, direction="write"),
                "messageBeforePaying": convert_and_respect_annotation_metadata(
                    object_=message_before_paying, annotation=LabelElement, direction="write"
                ),
                "notes": convert_and_respect_annotation_metadata(
                    object_=notes, annotation=NoteElement, direction="write"
                ),
                "page": convert_and_respect_annotation_metadata(
                    object_=page, annotation=PageElement, direction="write"
                ),
                "paymentButton": convert_and_respect_annotation_metadata(
                    object_=payment_button, annotation=LabelElement, direction="write"
                ),
                "paymentMethods": convert_and_respect_annotation_metadata(
                    object_=payment_methods, annotation=MethodElement, direction="write"
                ),
                "payor": convert_and_respect_annotation_metadata(
                    object_=payor, annotation=PayorElement, direction="write"
                ),
                "review": convert_and_respect_annotation_metadata(
                    object_=review, annotation=HeaderElement, direction="write"
                ),
                "settings": convert_and_respect_annotation_metadata(
                    object_=settings, annotation=PagelinkSetting, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_pay_link_from_id(
        self, pay_link_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Deletes a payment link by ID.

        Parameters
        ----------
        pay_link_id : str
            ID for the payment link.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentLink/{jsonable_encoder(pay_link_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_pay_link_from_id(
        self, paylink_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetPayLinkFromIdResponse]:
        """
        Retrieves a payment link by ID.

        Parameters
        ----------
        paylink_id : str
            ID for payment link

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetPayLinkFromIdResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentLink/load/{jsonable_encoder(paylink_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetPayLinkFromIdResponse,
                    parse_obj_as(
                        type_=GetPayLinkFromIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def push_pay_link_from_id(
        self, pay_link_id: str, *, request: PushPayLinkRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Send a payment link to the specified email addresses or phone numbers.

        Parameters
        ----------
        pay_link_id : str
            ID for the payment link.

        request : PushPayLinkRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentLink/push/{jsonable_encoder(pay_link_id)}",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=PushPayLinkRequest, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def refresh_pay_link_from_id(
        self,
        pay_link_id: str,
        *,
        amount_fixed: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Refresh a payment link's content after an update.

        Parameters
        ----------
        pay_link_id : str
            ID for the payment link.

        amount_fixed : typing.Optional[bool]
            Indicates whether customer can modify the payment amount. A value of `true` means the amount isn't modifiable, a value `false` means the payor can modify the amount to pay.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentLink/refresh/{jsonable_encoder(pay_link_id)}",
            method="GET",
            params={
                "amountFixed": amount_fixed,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def send_pay_link_from_id(
        self,
        pay_link_id: str,
        *,
        attachfile: typing.Optional[bool] = None,
        mail_2: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Sends a payment link to the specified email addresses.

        Parameters
        ----------
        pay_link_id : str
            ID for the payment link.

        attachfile : typing.Optional[bool]
            When `true`, attaches a PDF version of invoice to the email.

        mail_2 : typing.Optional[str]
            List of recipient email addresses. When there is more than one, separate them by a semicolon (;).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentLink/send/{jsonable_encoder(pay_link_id)}",
            method="GET",
            params={
                "attachfile": attachfile,
                "mail2": mail_2,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_pay_link_from_id(
        self,
        pay_link_id: str,
        *,
        contact_us: typing.Optional[ContactElement] = OMIT,
        logo: typing.Optional[Element] = OMIT,
        message_before_paying: typing.Optional[LabelElement] = OMIT,
        notes: typing.Optional[NoteElement] = OMIT,
        page: typing.Optional[PageElement] = OMIT,
        payment_button: typing.Optional[LabelElement] = OMIT,
        payment_methods: typing.Optional[MethodElement] = OMIT,
        review: typing.Optional[HeaderElement] = OMIT,
        settings: typing.Optional[PagelinkSetting] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Updates a payment link's details.

        Parameters
        ----------
        pay_link_id : str
            ID for the payment link.

        contact_us : typing.Optional[ContactElement]
            ContactUs section of payment link page

        logo : typing.Optional[Element]
            Logo section of payment link page

        message_before_paying : typing.Optional[LabelElement]
            Message section of payment link page

        notes : typing.Optional[NoteElement]
            Notes section of payment link page

        page : typing.Optional[PageElement]
            Page header section of payment link page

        payment_button : typing.Optional[LabelElement]
            Payment button section of payment link page

        payment_methods : typing.Optional[MethodElement]
            Payment methods section of payment link page

        review : typing.Optional[HeaderElement]
            Review section of payment link page

        settings : typing.Optional[PagelinkSetting]
            Settings section of payment link page

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentLink/update/{jsonable_encoder(pay_link_id)}",
            method="PUT",
            json={
                "contactUs": convert_and_respect_annotation_metadata(
                    object_=contact_us, annotation=ContactElement, direction="write"
                ),
                "logo": convert_and_respect_annotation_metadata(object_=logo, annotation=Element, direction="write"),
                "messageBeforePaying": convert_and_respect_annotation_metadata(
                    object_=message_before_paying, annotation=LabelElement, direction="write"
                ),
                "notes": convert_and_respect_annotation_metadata(
                    object_=notes, annotation=NoteElement, direction="write"
                ),
                "page": convert_and_respect_annotation_metadata(
                    object_=page, annotation=PageElement, direction="write"
                ),
                "paymentButton": convert_and_respect_annotation_metadata(
                    object_=payment_button, annotation=LabelElement, direction="write"
                ),
                "paymentMethods": convert_and_respect_annotation_metadata(
                    object_=payment_methods, annotation=MethodElement, direction="write"
                ),
                "review": convert_and_respect_annotation_metadata(
                    object_=review, annotation=HeaderElement, direction="write"
                ),
                "settings": convert_and_respect_annotation_metadata(
                    object_=settings, annotation=PagelinkSetting, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def add_pay_link_from_bill_lot_number(
        self,
        lot_number: str,
        *,
        entry_point: Entry,
        vendor_number: str,
        mail_2: typing.Optional[str] = None,
        amount_fixed: typing.Optional[str] = None,
        contact_us: typing.Optional[ContactElement] = OMIT,
        invoices: typing.Optional[InvoiceElement] = OMIT,
        logo: typing.Optional[Element] = OMIT,
        message_before_paying: typing.Optional[LabelElement] = OMIT,
        notes: typing.Optional[NoteElement] = OMIT,
        page: typing.Optional[PageElement] = OMIT,
        payment_button: typing.Optional[LabelElement] = OMIT,
        payment_methods: typing.Optional[MethodElement] = OMIT,
        payor: typing.Optional[PayorElement] = OMIT,
        review: typing.Optional[HeaderElement] = OMIT,
        settings: typing.Optional[PagelinkSetting] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponsePaymentLinks]:
        """
        Generates a vendor payment link for a specific bill lot number. This allows you to pay all bills with the same lot number for a vendor with a single payment link.

        Parameters
        ----------
        lot_number : str
            Lot number of the bills to pay. All bills with this lot number will be included.

        entry_point : Entry

        vendor_number : str
            The vendor number for the vendor being paid with this payment link.

        mail_2 : typing.Optional[str]
            List of recipient email addresses. When there is more than one, separate them by a semicolon (;).

        amount_fixed : typing.Optional[str]
            Indicates whether customer can modify the payment amount. A value of `true` means the amount isn't modifiable, a value `false` means the payor can modify the amount to pay.

        contact_us : typing.Optional[ContactElement]
            ContactUs section of payment link page

        invoices : typing.Optional[InvoiceElement]
            Invoices section of payment link page

        logo : typing.Optional[Element]
            Logo section of payment link page

        message_before_paying : typing.Optional[LabelElement]
            Message section of payment link page

        notes : typing.Optional[NoteElement]
            Notes section of payment link page

        page : typing.Optional[PageElement]
            Page header section of payment link page

        payment_button : typing.Optional[LabelElement]
            Payment button section of payment link page

        payment_methods : typing.Optional[MethodElement]
            Payment methods section of payment link page

        payor : typing.Optional[PayorElement]
            Customer/Payor section of payment link page

        review : typing.Optional[HeaderElement]
            Review section of payment link page

        settings : typing.Optional[PagelinkSetting]
            Settings section of payment link page

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponsePaymentLinks]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"PaymentLink/bill/lotNumber/{jsonable_encoder(lot_number)}",
            method="POST",
            params={
                "entryPoint": entry_point,
                "vendorNumber": vendor_number,
                "mail2": mail_2,
                "amountFixed": amount_fixed,
            },
            json={
                "contactUs": convert_and_respect_annotation_metadata(
                    object_=contact_us, annotation=ContactElement, direction="write"
                ),
                "invoices": convert_and_respect_annotation_metadata(
                    object_=invoices, annotation=InvoiceElement, direction="write"
                ),
                "logo": convert_and_respect_annotation_metadata(object_=logo, annotation=Element, direction="write"),
                "messageBeforePaying": convert_and_respect_annotation_metadata(
                    object_=message_before_paying, annotation=LabelElement, direction="write"
                ),
                "notes": convert_and_respect_annotation_metadata(
                    object_=notes, annotation=NoteElement, direction="write"
                ),
                "page": convert_and_respect_annotation_metadata(
                    object_=page, annotation=PageElement, direction="write"
                ),
                "paymentButton": convert_and_respect_annotation_metadata(
                    object_=payment_button, annotation=LabelElement, direction="write"
                ),
                "paymentMethods": convert_and_respect_annotation_metadata(
                    object_=payment_methods, annotation=MethodElement, direction="write"
                ),
                "payor": convert_and_respect_annotation_metadata(
                    object_=payor, annotation=PayorElement, direction="write"
                ),
                "review": convert_and_respect_annotation_metadata(
                    object_=review, annotation=HeaderElement, direction="write"
                ),
                "settings": convert_and_respect_annotation_metadata(
                    object_=settings, annotation=PagelinkSetting, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymentLinks,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymentLinks,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

MethodElementSettingsApplePayButtonType = typing.Union[
    typing.Literal[
        "plain",
        "buy",
        "donate",
        "check-out",
        "book",
        "continue",
        "top-up",
        "order",
        "rent",
        "support",
        "contribute",
        "tip",
        "pay",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .card_acceptance_element import CardAcceptanceElement
from .card_fee_section import CardFeeSection
from .card_flat_section import CardFlatSection
from .card_icp_section import CardIcpSection
from .card_pass_through_section import CardPassThroughSection
from .template_element import TemplateElement
from .visible import Visible


class CardService(UniversalBaseModel):
    batch_cutoff_time: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="batchCutoffTime")
    ] = pydantic.Field(default=None)
    """
    Controls how to present the `batchCutoffTime` field on the application. If this field isn't sent, batch cut off time defaults to 5 ET.
    """

    card_acceptance: typing_extensions.Annotated[
        typing.Optional[CardAcceptanceElement], FieldMetadata(alias="cardAcceptance")
    ] = None
    card_fees: typing_extensions.Annotated[typing.Optional[CardFeeSection], FieldMetadata(alias="cardFees")] = None
    card_flat: typing_extensions.Annotated[typing.Optional[CardFlatSection], FieldMetadata(alias="cardFlat")] = None
    card_flat_amountx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardFlat_amountxAuth")
    ] = None
    card_flat_high_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardFlat_highPayRange")
    ] = None
    card_flat_low_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardFlat_lowPayRange")
    ] = None
    card_flat_percentx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardFlat_percentxAuth")
    ] = None
    card_icp: typing_extensions.Annotated[typing.Optional[CardIcpSection], FieldMetadata(alias="cardICP")] = None
    card_icp_amountx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardICP_amountxAuth")
    ] = None
    card_icp_high_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardICP_highPayRange")
    ] = None
    card_icp_low_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardICP_lowPayRange")
    ] = None
    card_icp_percentx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardICP_percentxAuth")
    ] = None
    card_pass_through: typing_extensions.Annotated[
        typing.Optional[CardPassThroughSection], FieldMetadata(alias="cardPassThrough")
    ] = None
    card_pass_through_amount_recurring: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardPassThrough_amountRecurring")
    ] = None
    card_pass_through_amountx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardPassThrough_amountxAuth")
    ] = None
    card_pass_through_high_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardPassThrough_highPayRange")
    ] = None
    card_pass_through_low_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardPassThrough_lowPayRange")
    ] = None
    card_pass_through_percent_recurring: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardPassThrough_percentRecurring")
    ] = None
    card_pass_through_percentx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="cardPassThrough_percentxAuth")
    ] = None
    discount_frequency: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="discountFrequency")
    ] = None
    funding_rollup: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="fundingRollup")
    ] = None
    gateway: typing.Optional[TemplateElement] = None
    pass_through_cost: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="passThroughCost")
    ] = None
    pdf_template_id: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="pdfTemplateId")
    ] = None
    pricing_plan: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="pricingPlan")] = None
    pricing_type: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="pricingType")] = (
        None
    )
    processor: typing.Optional[TemplateElement] = None
    provider: typing.Optional[TemplateElement] = None
    tier_name: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="tierName")] = None
    visible: typing.Optional[Visible] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .file_content_image_only import FileContentImageOnly
    from .ocr_attachment import OcrAttachment
    from .ocr_bill_item import OcrBillItem
    from .ocr_bill_item_additional_data import OcrBillItemAdditionalData
    from .ocr_response_data import OcrResponseData
    from .ocr_result_data import OcrResultData
    from .ocr_vendor import OcrVendor
    from .ocr_vendor_additional_data import OcrVendorAdditionalData
    from .ocr_vendor_billing_data import OcrVendorBillingData
    from .payabli_api_response_ocr import PayabliApiResponseOcr
    from .type_result import TypeResult
_dynamic_imports: typing.Dict[str, str] = {
    "FileContentImageOnly": ".file_content_image_only",
    "OcrAttachment": ".ocr_attachment",
    "OcrBillItem": ".ocr_bill_item",
    "OcrBillItemAdditionalData": ".ocr_bill_item_additional_data",
    "OcrResponseData": ".ocr_response_data",
    "OcrResultData": ".ocr_result_data",
    "OcrVendor": ".ocr_vendor",
    "OcrVendorAdditionalData": ".ocr_vendor_additional_data",
    "OcrVendorBillingData": ".ocr_vendor_billing_data",
    "PayabliApiResponseOcr": ".payabli_api_response_ocr",
    "TypeResult": ".type_result",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "FileContentImageOnly",
    "OcrAttachment",
    "OcrBillItem",
    "OcrBillItemAdditionalData",
    "OcrResponseData",
    "OcrResultData",
    "OcrVendor",
    "OcrVendorAdditionalData",
    "OcrVendorBillingData",
    "PayabliApiResponseOcr",
    "TypeResult",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.customer_id import CustomerId
from ...types.is_success import IsSuccess
from ...types.response_text import ResponseText


class UpdateSubscriptionResponse(UniversalBaseModel):
    """
    Success response
    """

    is_success: typing_extensions.Annotated[typing.Optional[IsSuccess], FieldMetadata(alias="isSuccess")] = None
    response_data: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="responseData")] = (
        pydantic.Field(default=None)
    )
    """
    
    If `isSuccess` = true, this contains the identifier of the subscription, and sometimes extra information, depending on what was updated.
    
    If `isSuccess` = false, this contains the reason for the failure.
    """

    response_text: typing_extensions.Annotated[ResponseText, FieldMetadata(alias="responseText")]
    customer_id: typing_extensions.Annotated[typing.Optional[CustomerId], FieldMetadata(alias="customerId")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data import AdditionalData
from .address_addtl_nullable import AddressAddtlNullable
from .address_nullable import AddressNullable
from .billing_data_response import BillingDataResponse
from .city_nullable import CityNullable
from .contacts_response import ContactsResponse
from .created_at import CreatedAt
from .dbaname import Dbaname
from .ein import Ein
from .email import Email
from .enrollment_status import EnrollmentStatus
from .entrypointfield import Entrypointfield
from .external_paypoint_id import ExternalPaypointId
from .internal_reference_id import InternalReferenceId
from .last_modified import LastModified
from .legalname import Legalname
from .mcc import Mcc
from .org_parent_id import OrgParentId
from .org_parent_name import OrgParentName
from .payee_name import PayeeName
from .remit_email import RemitEmail
from .remitaddress_1 import Remitaddress1
from .remitaddress_2 import Remitaddress2
from .remitcity import Remitcity
from .remitcountry import Remitcountry
from .remitstate import Remitstate
from .remitzip import Remitzip
from .state_nullable import StateNullable
from .vendor_number import VendorNumber
from .vendor_payment_method_string import VendorPaymentMethodString
from .vendor_response_stored_method import VendorResponseStoredMethod
from .vendor_summary import VendorSummary
from .vendorid import Vendorid
from .vendorstatus import Vendorstatus
from .zip import Zip


class VendorQueryRecord(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from payabli import (
        BillingDataResponse,
        ContactsResponse,
        VendorQueryRecord,
        VendorSummary,
    )

    VendorQueryRecord(
        vendor_number="1234",
        name_1="Herman's Coatings",
        name_2="Herman's Coating Supply Company, LLC",
        ein="123456789",
        phone="212-555-1234",
        email="example@email.com",
        address_1="123 Ocean Drive",
        address_2="Suite 400",
        city="Bristol",
        state="GA",
        zip="31113",
        country="US",
        mcc="7777",
        location_code="LOC123",
        contacts=ContactsResponse(
            contact_email="eric@martinezcoatings.com",
            contact_name="Eric Martinez",
            contact_phone="5555555555",
            contact_title="Owner",
        ),
        billing_data=BillingDataResponse(
            id=123456,
            account_id="bank-account-001",
            nickname="Main Checking Account",
            bank_name="Example Bank",
            routing_account="123456789",
            account_number="9876543210",
            type_account="Checking",
            bank_account_holder_name="John Doe",
            bank_account_holder_type="Business",
            bank_account_function=2,
            verified=True,
            status=1,
            services=[],
            default=True,
        ),
        vendor_status=1,
        vendor_id=1,
        summary=VendorSummary(
            active_bills=2,
            pending_bills=4,
            in_transit_bills=3,
            paid_bills=18,
            overdue_bills=1,
            approved_bills=5,
            disapproved_bills=1,
            total_bills=34,
            active_bills_amount=1250.75,
            pending_bills_amount=2890.5,
            in_transit_bills_amount=1675.25,
            paid_bills_amount=15420.8,
            overdue_bills_amount=425.0,
            approved_bills_amount=3240.9,
            disapproved_bills_amount=180.0,
            total_bills_amount=25083.2,
        ),
        paypoint_legalname="Sunshine Services, LLC",
        paypoint_dbaname="Sunshine Gutters",
        paypoint_entryname="d193cf9a46",
        parent_org_name="PropertyManager Pro",
        parent_org_id=1000,
        created_date=datetime.datetime.fromisoformat(
            "2022-07-01 15:00:01+00:00",
        ),
        last_updated=datetime.datetime.fromisoformat(
            "2022-07-01 15:00:01+00:00",
        ),
        remit_address_1="123 Walnut Street",
        remit_address_2="Suite 900",
        remit_city="Miami",
        remit_state="FL",
        remit_zip="31113",
        remit_country="US",
        payee_name_1="payeeName1",
        payee_name_2="payeeName2",
        custom_field_1="",
        custom_field_2="",
        customer_vendor_account="123-456",
        internal_reference_id=1000000,
        external_paypoint_id="Paypoint-100",
        stored_methods=[],
    )
    """

    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalData], FieldMetadata(alias="additionalData")
    ] = None
    address_1: typing_extensions.Annotated[typing.Optional[AddressNullable], FieldMetadata(alias="Address1")] = None
    address_2: typing_extensions.Annotated[typing.Optional[AddressAddtlNullable], FieldMetadata(alias="Address2")] = (
        None
    )
    billing_data: typing_extensions.Annotated[
        typing.Optional[BillingDataResponse], FieldMetadata(alias="BillingData")
    ] = None
    city: typing_extensions.Annotated[typing.Optional[CityNullable], FieldMetadata(alias="City")] = None
    contacts: typing_extensions.Annotated[typing.Optional[ContactsResponse], FieldMetadata(alias="Contacts")] = None
    country: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Country")] = None
    created_date: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="CreatedDate")] = None
    customer_vendor_account: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="customerVendorAccount")
    ] = None
    custom_field_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customField1")] = None
    custom_field_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customField2")] = None
    ein: typing_extensions.Annotated[typing.Optional[Ein], FieldMetadata(alias="EIN")] = None
    email: typing_extensions.Annotated[typing.Optional[Email], FieldMetadata(alias="Email")] = None
    enrollment_status: typing_extensions.Annotated[
        typing.Optional[EnrollmentStatus], FieldMetadata(alias="EnrollmentStatus")
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    internal_reference_id: typing_extensions.Annotated[
        typing.Optional[InternalReferenceId], FieldMetadata(alias="InternalReferenceId")
    ] = None
    last_updated: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="LastUpdated")] = None
    location_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="LocationCode")] = None
    mcc: typing_extensions.Annotated[typing.Optional[Mcc], FieldMetadata(alias="Mcc")] = None
    name_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Name1")] = None
    name_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Name2")] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    parent_org_id: typing_extensions.Annotated[typing.Optional[OrgParentId], FieldMetadata(alias="ParentOrgId")] = None
    payee_name_1: typing_extensions.Annotated[typing.Optional[PayeeName], FieldMetadata(alias="payeeName1")] = None
    payee_name_2: typing_extensions.Annotated[typing.Optional[PayeeName], FieldMetadata(alias="payeeName2")] = None
    payment_method: typing_extensions.Annotated[
        typing.Optional[VendorPaymentMethodString], FieldMetadata(alias="PaymentMethod")
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        None
    )
    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield], FieldMetadata(alias="PaypointEntryname")
    ] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = None
    phone: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Phone")] = None
    remit_address_1: typing_extensions.Annotated[
        typing.Optional[Remitaddress1], FieldMetadata(alias="remitAddress1")
    ] = None
    remit_address_2: typing_extensions.Annotated[
        typing.Optional[Remitaddress2], FieldMetadata(alias="remitAddress2")
    ] = None
    remit_city: typing_extensions.Annotated[typing.Optional[Remitcity], FieldMetadata(alias="remitCity")] = None
    remit_country: typing_extensions.Annotated[typing.Optional[Remitcountry], FieldMetadata(alias="remitCountry")] = (
        None
    )
    remit_email: typing_extensions.Annotated[typing.Optional[RemitEmail], FieldMetadata(alias="RemitEmail")] = None
    remit_state: typing_extensions.Annotated[typing.Optional[Remitstate], FieldMetadata(alias="remitState")] = None
    remit_zip: typing_extensions.Annotated[typing.Optional[Remitzip], FieldMetadata(alias="remitZip")] = None
    state: typing_extensions.Annotated[typing.Optional[StateNullable], FieldMetadata(alias="State")] = None
    stored_methods: typing_extensions.Annotated[
        typing.Optional[typing.List[VendorResponseStoredMethod]], FieldMetadata(alias="StoredMethods")
    ] = None
    summary: typing_extensions.Annotated[typing.Optional[VendorSummary], FieldMetadata(alias="Summary")] = None
    vendor_id: typing_extensions.Annotated[typing.Optional[Vendorid], FieldMetadata(alias="VendorId")] = None
    vendor_number: typing_extensions.Annotated[typing.Optional[VendorNumber], FieldMetadata(alias="VendorNumber")] = (
        None
    )
    vendor_status: typing_extensions.Annotated[typing.Optional[Vendorstatus], FieldMetadata(alias="VendorStatus")] = (
        None
    )
    zip: typing_extensions.Annotated[typing.Optional[Zip], FieldMetadata(alias="Zip")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ach_holder import AchHolder
from .ach_sec_code import AchSecCode
from .achaccount import Achaccount
from .achaccounttype import Achaccounttype
from .achrouting import Achrouting
from .cardcvv import Cardcvv
from .cardexp import Cardexp
from .cardholder import Cardholder
from .cardnumber import Cardnumber
from .cardzip import Cardzip
from .device import Device
from .initiator import Initiator
from .methodall import Methodall
from .save_if_success import SaveIfSuccess
from .stored_method_usage_type import StoredMethodUsageType
from .storedmethodid import Storedmethodid


class PayMethodBodyAllFields(UniversalBaseModel):
    """
    Model for the PaymentMethod object, includes all method types.
    """

    ach_account: typing_extensions.Annotated[Achaccount, FieldMetadata(alias="achAccount")] = pydantic.Field()
    """
    Bank account number. This field is **required** when method = 'ach'.
    """

    ach_account_type: typing_extensions.Annotated[
        typing.Optional[Achaccounttype], FieldMetadata(alias="achAccountType")
    ] = None
    ach_code: typing_extensions.Annotated[typing.Optional[AchSecCode], FieldMetadata(alias="achCode")] = None
    ach_holder: typing_extensions.Annotated[AchHolder, FieldMetadata(alias="achHolder")]
    ach_routing: typing_extensions.Annotated[Achrouting, FieldMetadata(alias="achRouting")] = pydantic.Field()
    """
    ABA/routing number of Bank account. This field is **required** when method = 'ach'.
    """

    cardcvv: typing.Optional[Cardcvv] = None
    cardexp: typing.Optional[Cardexp] = None
    card_holder: typing_extensions.Annotated[typing.Optional[Cardholder], FieldMetadata(alias="cardHolder")] = None
    cardnumber: typing.Optional[Cardnumber] = None
    cardzip: typing.Optional[Cardzip] = None
    device: typing.Optional[Device] = None
    initator: typing.Optional[Initiator] = None
    method: typing.Optional[Methodall] = None
    save_if_success: typing_extensions.Annotated[
        typing.Optional[SaveIfSuccess], FieldMetadata(alias="saveIfSuccess")
    ] = None
    stored_method_id: typing_extensions.Annotated[
        typing.Optional[Storedmethodid], FieldMetadata(alias="storedMethodId")
    ] = None
    stored_method_usage_type: typing_extensions.Annotated[
        typing.Optional[StoredMethodUsageType], FieldMetadata(alias="storedMethodUsageType")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.authcode import Authcode
from ...types.avsresponsetext import Avsresponsetext
from ...types.customeridtrans import Customeridtrans
from ...types.cvvresponsetext import Cvvresponsetext
from ...types.method_reference_id import MethodReferenceId
from ...types.referenceidtrans import Referenceidtrans
from ...types.result_code import ResultCode
from ...types.resulttext import Resulttext


class VoidResponseData(UniversalBaseModel):
    """
    Response data for void transactions
    """

    auth_code: typing_extensions.Annotated[Authcode, FieldMetadata(alias="authCode")]
    reference_id: typing_extensions.Annotated[Referenceidtrans, FieldMetadata(alias="referenceId")]
    result_code: typing_extensions.Annotated[ResultCode, FieldMetadata(alias="resultCode")]
    result_text: typing_extensions.Annotated[Resulttext, FieldMetadata(alias="resultText")]
    avs_response_text: typing_extensions.Annotated[
        typing.Optional[Avsresponsetext], FieldMetadata(alias="avsResponseText")
    ] = None
    cvv_response_text: typing_extensions.Annotated[
        typing.Optional[Cvvresponsetext], FieldMetadata(alias="cvvResponseText")
    ] = None
    customer_id: typing_extensions.Annotated[typing.Optional[Customeridtrans], FieldMetadata(alias="customerId")] = None
    method_reference_id: typing_extensions.Annotated[
        typing.Optional[MethodReferenceId], FieldMetadata(alias="methodReferenceId")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .enabled import Enabled


class ServiceCost(UniversalBaseModel):
    description: typing.Optional[str] = None
    enabled: typing.Optional[Enabled] = None
    monthly_cost: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="monthlyCost")] = None
    name: typing.Optional[str] = None
    reseller: typing.Optional[bool] = None
    setup_cost: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="setupCost")] = None
    tx_cost: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="txCost")] = None
    tx_percent_cost: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="txPercentCost")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .payment_categories import PaymentCategories
from .split_funding_refund_content import SplitFundingRefundContent


class RefundDetail(UniversalBaseModel):
    """
    Object containing details about the refund, including line items and optional split instructions.
    """

    categories: typing.Optional[typing.List[PaymentCategories]] = pydantic.Field(default=None)
    """
    Array of payment categories/line items describing the amount to be paid. Note: These categories are for information only and aren't validated against the total amount provided.
    """

    split_refunding: typing_extensions.Annotated[
        typing.Optional[typing.List[SplitFundingRefundContent]], FieldMetadata(alias="splitRefunding")
    ] = pydantic.Field(default=None)
    """
    Array of objects containing split instructions for the refund.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class ChargebackMessage(UniversalBaseModel):
    id: typing_extensions.Annotated[int, FieldMetadata(alias="Id")] = pydantic.Field()
    """
    Message identifier.
    """

    room_id: typing_extensions.Annotated[int, FieldMetadata(alias="RoomId")] = pydantic.Field()
    """
    Room identifier for the message.
    """

    user_id: typing_extensions.Annotated[int, FieldMetadata(alias="UserId")] = pydantic.Field()
    """
    User identifier who sent the message.
    """

    user_name: typing_extensions.Annotated[str, FieldMetadata(alias="UserName")] = pydantic.Field()
    """
    Name of the user who sent the message.
    """

    content: typing_extensions.Annotated[str, FieldMetadata(alias="Content")] = pydantic.Field()
    """
    Content of the message.
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="CreatedAt")] = pydantic.Field()
    """
    Timestamp when the message was created.
    """

    message_type: typing_extensions.Annotated[int, FieldMetadata(alias="MessageType")] = pydantic.Field()
    """
    Type of message.
    """

    message_properties: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]], FieldMetadata(alias="MessageProperties")
    ] = pydantic.Field(default=None)
    """
    Additional properties of the message.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BillDataPaymentTerms = typing.Union[
    typing.Literal[
        "PIA",
        "CIA",
        "UR",
        "NET10",
        "NET20",
        "NET30",
        "NET45",
        "NET60",
        "NET90",
        "EOM",
        "MFI",
        "5MFI",
        "10MFI",
        "15MFI",
        "20MFI",
        "2/10NET30",
        "UF",
        "10UF",
        "20UF",
        "25UF",
        "50UF",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.cardcvv import Cardcvv
from ...types.cardexp import Cardexp
from ...types.cardholder import Cardholder
from ...types.cardnumber import Cardnumber
from ...types.cardzip import Cardzip


class TokenizeCard(UniversalBaseModel):
    method: str = pydantic.Field()
    """
    The type of payment method to tokenize. For cards, this is always `card`.
    """

    cardcvv: typing.Optional[Cardcvv] = None
    cardexp: Cardexp
    card_holder: typing_extensions.Annotated[Cardholder, FieldMetadata(alias="cardHolder")]
    cardnumber: Cardnumber
    cardzip: typing.Optional[Cardzip] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PoiDevice(UniversalBaseModel):
    """
    Information about the point of interaction device (also known as a terminal or cloud device) used to process the transaction.
    """

    connected: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The device connection status.
    """

    date_de_registered: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="dateDeRegistered")
    ] = pydantic.Field(default=None)
    """
    The date the device was unregistered.
    """

    date_registered: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="dateRegistered")
    ] = pydantic.Field(default=None)
    """
    The date the device was registered.
    """

    device_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="deviceId")] = pydantic.Field(
        default=None
    )
    """
    The device identifier.
    """

    device_license: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="deviceLicense")] = (
        pydantic.Field(default=None)
    )
    """
    Device license. This is typically the same as `deviceId`.
    """

    device_nick_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="deviceNickName")] = (
        pydantic.Field(default=None)
    )
    """
    Device description provided during registration.
    """

    last_connected_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="lastConnectedDate")
    ] = pydantic.Field(default=None)
    """
    Last connected date.
    """

    last_disconnected_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="lastDisconnectedDate")
    ] = pydantic.Field(default=None)
    """
    Last disconnected date.
    """

    last_transaction_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime], FieldMetadata(alias="lastTransactionDate")
    ] = pydantic.Field(default=None)
    """
    Last transaction date.
    """

    make: typing.Optional[str] = pydantic.Field(default=None)
    """
    The device manufacturer.
    """

    model: typing.Optional[str] = pydantic.Field(default=None)
    """
    The device model.
    """

    registered: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The device registration status.
    """

    serial_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="serialNumber")] = (
        pydantic.Field(default=None)
    )
    """
    The device serial number.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .page_identifier import PageIdentifier
from .payabli_api_response_error_400_response_data import PayabliApiResponseError400ResponseData


class PayabliApiResponseError400(UniversalBaseModel):
    is_success: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="isSuccess")] = pydantic.Field(
        default=None
    )
    """
    Boolean indicating whether the operation was successful. A `true` value indicates success. A `false` value indicates failure.
    """

    pageidentifier: typing.Optional[PageIdentifier] = None
    response_code: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="responseCode")] = (
        pydantic.Field(default=None)
    )
    """
    A code that indicates the operation's failure reason. See [API Response Codes](https://docs.payabli.com/api-reference/api-responses) for a full reference.
    """

    response_data: typing_extensions.Annotated[
        typing.Optional[PayabliApiResponseError400ResponseData], FieldMetadata(alias="responseData")
    ] = pydantic.Field(default=None)
    """
    Describes the reason for a failed operation and how to resolve it.
    """

    response_text: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="responseText")] = (
        pydantic.Field(default=None)
    )
    """
    Response text for operation: 'Success' or 'Declined'.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
